@@ -29,6 +29,7 @@ from common import *
 import fio
 from transducers import SFT, infixTransducer, prefixTransducer, suffixTransducer, outfixTransducer, hypercodeTransducer
 import fa
+import fl
 
 
 class CodeProperty(object):
@@ -790,3 +791,113 @@ def exponentialDensityP(aut):
                 if not levelSym[j][0] == pattern[j % d]:
                     return True
     return False
+
+
+def long2base(n, q):
+    """ Maps n to a list of digits corresponding to the base q representation of n in reverse order
+
+    :param int n: a positive integer
+    :param int q: base to represent n
+    :return: list of q-ary 'digits', that is, elements of {0,1,..,q-1}
+    :rtype: list"""
+    lt = []
+    while n > 0:
+        lt.append(n % q)
+        n /= q
+    return lt
+
+
+def list2string(lt, dy):
+    """
+    :param list lt: list of nonnegative integers from some set {0,1,...,q-1}
+    :param dict dy: mapping from {0,1,...,q-1} to some alphabet symbols
+    :return: string of symbols corresponding to the integers in lt
+    :rtype: str"""
+    return "".join([dy[i] for i in lt])
+
+
+def notUniversalStatW(a, l, maxIter=20000):
+    """ Tests statistically whether the NFA a is l-non-universal, by evaluating a on <= maxIter randomly chosen words of length l
+
+    :type a: NFA
+    :param int l: nonnegative integer
+    :type maxIter: int
+    :return: (w,i) where w is the word found at i-th try; or (None, i) after i tries
+    :rtype: tuple"""
+    if l == 0:  # first handle the case of the empty word
+        if a.evalWordP(Epsilon):
+            return None, 1
+        else:
+            return Epsilon, 1
+    lt = [s for s in a.Sigma]
+    # dictionary dy is used to refer to each alphabet symbol in lt using a numeric index
+    dy = {i: lt[i] for i in range(0, len(lt))}
+    q = len(lt)
+    numWords = q ** l
+    tmp = set()  # will remember in tmp the distinct words generated so far
+    random.seed()
+    for j in range(0, maxIter):
+        # next pick one word out of the possible numWords of length l; specifically pick
+        # the i-th one, where i is randomly chosen. For this, choose a random int n whose
+        # base q representation is of the form '1d_1...d_l', where the l d's are digits
+        # in {0,1,...,q-1}, so the desired number i is the one represented as 'd_1...d_l'
+        n = random.randint(numWords, 2 * numWords - 1)
+        w = list2string(long2base(n, q), dy)[:l]
+        if not a.evalWordP(w):
+            return w, j + 1
+        tmp.add(w)  # Note: the current random word could be equal to a past one !!
+        if len(tmp) == numWords:
+            return None, numWords
+    return None, maxIter
+
+
+def symmAndRefl(t, ipt=False):
+    """ Return the  transducer      t | t.inverse,  if ipt is True;
+        return the transducer  t | t.inverse | id,  otherwise
+
+    :type t: SFT
+    :type ipt: bool
+    :rtype: SFT"""
+    t1 = t | t.inverse()
+    if ipt is True:
+        return t1
+    t2 = SFT()
+    s0 = t2.addState()
+    t2.addInitial(s0)
+    t2.addFinal(s0)
+    for sym in t.Sigma:
+        t2.addTransition(s0, sym, sym, s0)
+    return t1 | t2
+
+
+def constructCode(n, l, p, ipt=False, seed=None):
+    """ Returns up to n words of length l satisfying the property p, the first one being seed.
+        If ipt is True, the property is assumed to be input-preserving transducer type
+
+    :type p: IATProp|IPTProp
+    :type l: int
+    :type n: int
+    :type seed: str
+    :type ipt: bool
+    :rtype: list"""
+    if n == 0:
+        return []
+    if l == 0:
+        return [Epsilon]
+    t = symmAndRefl(p.Aut, ipt)  # make a version of p.Aut that is symmetric and reflexive
+    if seed is None or len(seed) != l:
+        a0 = list(t.Sigma)[0]
+        seed = "".join([a0 for _ in range(l)])
+    lt = [seed]
+    cnt, more = 1, True
+    while more and cnt < n:
+        a = fl.FL(lt).trieFA().toNFA()
+        b = t.runOnNFA(a)
+        (w, i) = notUniversalStatW(b, l)
+        if w is None:
+            more = False
+        else:
+            cnt += 1
+            lt.append(w)
+    return lt
+
