@@ -26,7 +26,7 @@ Finite languages manipulation
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA."""
 
 import fa
-from copy import copy
+from copy import copy, deepcopy
 from common import *
 import random
 
@@ -425,6 +425,20 @@ class ADFA(fa.DFA, AFA):
             foo.complete()
         return len(foo) == len(self)
 
+    def forceToDCFA(self):
+        """ Conversion to DCFA
+
+        :rtype: DFA"""
+        new = fa.DFA()
+        new.States = deepcopy(self.States)
+        new.Sigma = deepcopy(self.Sigma)
+        new.Initial = self.Initial
+        new.Final = copy(self.Final)
+        for s in self.delta:
+            for c in self.delta[s]:
+                new.addTransition(s, c, self.delta[s][c])
+        return new
+
     def minimal(self):
         """Finds the minimal equivalent ADFA
 
@@ -571,6 +585,7 @@ class ADFA(fa.DFA, AFA):
         for (a, b) in toMerge:
             new.mergeStates(b, a)
         new.trim()
+        new = new.forceToDCFA()
         new.Length = l
         return new
 
