@@ -214,19 +214,21 @@ class xre(reex.regexp):
     
         :param reg: regular expression
         :rtype: bool """
-        s = [(self, reg)]
-        h = {(self, reg)}
+        s = {(self, reg)}
+        # h = {(self, reg)}
+        h = set([])
         sigma = self.setOfSymbols().union(reg.setOfSymbols())
         while s:
             s1, s2 = s.pop()
             if s1.ewp() != s2.ewp():
                 return False
+            h.add((s1, s2))
             for a in sigma:
                 der1 = s1.derivative(a)
                 der2 = s2.derivative(a)
                 if (der1, der2) not in h:
-                    h.add((der1, der2))
-                    s.append((der1, der2))
+                    #h.add((der1,der2))
+                    s.add((der1, der2))
         return True
 
     def toDFA(self, dfa_method="dfaDerivatives"):
@@ -322,6 +324,9 @@ class xre(reex.regexp):
         aut.addInitial(i)
         stack = [(self, i)]
         sigma = copy.deepcopy(self.setOfSymbols())
+        if self.Sigma is not None:
+            sigma |= self.Sigma
+        aut.setSigma(sigma)
         while stack:
             state, idx = stack.pop()
             state.setSigma(sigma)
@@ -390,9 +395,75 @@ class xre(reex.regexp):
         return s
 
 
-class xspecialConst(reex.specialConstant, xre):
+class xspecialConst(xre):
     """Base class for xepsilon and xempty."""
 
+    def __init__(self, sigma=None):
+        """
+        :param sigma: """
+        self.Sigma = sigma
+
+    def __copy__(self):
+        """
+        :return: """
+        return self
+
+    def __eq__(self, re):
+        """
+        :param re: the other re """
+        if type(self) == type(re):
+            return True
+
+    def setOfSymbols(self):
+        """
+        :return:"""
+        return set()
+
+    def alphabeticLength(self):
+        """
+        :return:"""
+        return 0
+
+    def _marked(self, pos):
+        """
+        :param pos:
+        :return:"""
+        return self, pos
+
+    def unmarked(self):
+        """The unmarked form of the regular expression. Each leaf in its syntactical tree becomes a regexp(),
+        the epsilon() or the emptyset().
+
+        :rtype: (general) regular expression"""
+        return self
+
+    def first(self):
+        """
+        :return: """
+        return []
+
+    def last(self):
+        """
+        :return:"""
+        return []
+
+    def followLists(self, lists=None):
+        """
+        :param lists:
+        :return:"""
+        if lists is None:
+            return {}
+        return lists
+
+    def followListsStar(self, lists=None):
+        """
+        :param lists:
+        :return:"""
+        if lists is None:
+            return {}
+        return lists
+
+
     def partialDerivatives(self, sigma):
         """
         :param sigma:
@@ -405,6 +476,17 @@ class xspecialConst(reex.specialConstant, xre):
         :return:"""
         return {xsigmaS(self.Sigma)}
 
+    def linearForm(self):
+        """
+        :return: dictionary mapping heads to sets of tails
+        :rtype: {symbol: set([regular expressions])}
+
+        .. seealso:: Antimirov, 95"""
+        if not self.Sigma:
+            syms = self.setOfSymbols()
+            self.setSigma(syms)
+        return self._linearForm()
+
     def _linearForm(self):
         """
         :return:"""
@@ -424,6 +506,11 @@ class xspecialConst(reex.specialConstant, xre):
         :return:"""
         return xempty(self.Sigma)
 
+    def support(self):
+        """
+        :return:"""
+        return {}
+
 
 class xempty(xspecialConst, reex.emptyset):
     """Class that represents the empty set."""
@@ -653,6 +740,13 @@ class xconnective(xre):
             s = s | i.setOfSymbols()
         return s
 
+    def _setSigma(self, s):
+        """
+        :return: """
+        for i in self.val:
+            i.setSigma(self.Sigma, s)
+
+
 
 class xdisj(xconnective):
     """Class that represents the disjunction operation."""
@@ -747,7 +841,7 @@ class xdisj(xconnective):
         elif len(s) == 1:
             return s.pop()
         else:
-            return xdisj(s)
+            return xdisj(s,self.Sigma)
 
     def partialDerivatives(self, sigma):
         """
@@ -870,7 +964,7 @@ class xconcat(xconnective):
         if len(self) == 2:
             return copy.deepcopy(self.val[1])
         else:
-            return xconcat(copy.deepcopy(self.val[1:]))
+            return xconcat(copy.deepcopy(self.val[1:]),self.Sigma)
 
     def headrev(self):
         """
@@ -883,7 +977,7 @@ class xconcat(xconnective):
         if len(self) == 2:
             return copy.deepcopy(self.val[0])
         else:
-            return xconcat(copy.deepcopy(self.val[:-1]))
+            return xconcat(copy.deepcopy(self.val[:-1]),self.Sigma)
 
     def ewp(self):
         """
@@ -901,7 +995,7 @@ class xconcat(xconnective):
         for i in self.val:
             mark, pos = i._marked(pos)
             l.append(mark)
-        return xconcat(l), pos
+        return xconcat(l,self.Sigma), pos
 
     def first(self):
         """
@@ -972,7 +1066,7 @@ class xconcat(xconnective):
                     bder.val.add(der)
                     return bder
                 else:
-                    return xdisj({der, bder})
+                    return xdisj({der, bder},self.Sigma)
         else:
             return der
 
@@ -1414,6 +1508,8 @@ class xstar(xre):
             else:
                 return xconj({self, sx})
 
+    def _setSigma(self, s):
+        self.val.setSigma(self.Sigma, s)
 
 class xnot(xre):
     """ xnot class """
@@ -1523,6 +1619,8 @@ class xnot(xre):
     def _not(self):
         return copy.deepcopy(self.val)
 
+    def _setSigma(self, s):
+        self.val.setSigma(self.Sigma, s)
 
 rpn = ["x -> {0:s} | i | +  x  x  | & x x | . x x  | ~ x | * x".format(Epsilon)]
 
@@ -1620,7 +1718,7 @@ def _disjrpn(arg1, arg2):
     if len(s) == 1:
         return s.pop()
     else:
-        return xdisj(s)
+        return xdisj(s,self.Sigma)
 
 
 def _conjrpn(arg1, arg2):
